/*
 * Client/server example courtesy of Marcia Perry.
 */

#include "CIF_Comm.H"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>


int main(int argc, char *argv[])
{
    const int MAX_MSG_SZ = 512;

    int	argn;
    int	timeout = -1;    
    CIF_Comm_Listener *cif_listener[3];
    CIF_Comm_Connection *cif_conn[3];
    CIF_Bool rc;
    CIF_Byte buf[80];
    char url[80];
    CIF_Comm_Message message;

    argn = 1;
    printf("CIF_Server: starting\n");
    while(argn < argc - 1)
    {
	if (strcmp(argv[argn], "-t") == 0)
	    timeout = atoi(argv[++argn]);
	else
	    fprintf(stderr, "unknown argument: %s\n\n", argv[argn]);
	argn++;
    }

    CIF_Comm::activate();

    strcpy(url, "x-cif-uum://224.35.36.37:5556/ttl=16+loopback=false");
    printf("CIF_Server: multicast connection for URL=%s\n",url);
    cif_conn[0] = CIF_Comm_Factory::get_Connection(url);
    assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
    assert(cif_conn[0] != CIF_NULL);
    printf("CIF_Server: multicast connection established\n");
     
    strcpy(url, "x-cif-udp:");
    printf("CIF_Server: establishing listener object for URL=%s\n",url);
    cif_listener[0] = CIF_Comm_Factory::get_Listener(url);
    assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
    assert(cif_listener[0] != CIF_NULL);
    printf("\nListening at %s\n\n",cif_listener[0]->get_URL());

    for (;;)
    {
        /* Wait for a remote process to contact the listener and 
	   form a new connection.  */
        printf("CIF_Server: waiting for connection--contact URL: %s\n\n",
	       cif_listener[0]->get_URL());

        if (timeout >= 0)
	{ 
	  printf("calling get_Connection with timeout %d\n",timeout);
	  cif_conn[1] = cif_listener[0]->get_Connection(timeout);
	}
	else
	{
	  printf("calling get_Connection with no timeout--blocks\n");
	  cif_conn[1] = cif_listener[0]->get_Connection();
	}
	    
	printf("get_Connection returned %p\n", cif_conn[1]);
	if (cif_conn[1] != CIF_NULL ||
	    CIF_Comm::status != CIF_Comm_Status::SUCCESS)
	{
	    printf("get_Listener() %s\n",
		   CIF_Comm::status.get_Description());
	}
	assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
	assert(cif_conn[1] != CIF_NULL);

	printf("CIF_Server: polling\n");
        while (cif_conn[1]->poll() == false)
	{
	   assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
	}

	rc = cif_conn[1]->receive(message);
	assert(rc == true);
	assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);

	printf("CIF_Server: rec'd msg: size=%d, tag=%d, data=%s\n",
	       message.get_Info().size,
	       message.get_Info().tag,
	       message.get_Data());

	printf("CIF_Server: sending multicast message\n");
        sprintf((char *)buf, "received %s\n", message.get_Data());
	rc = cif_conn[0]->send(buf,
			       strlen((char *)buf)+1,
			       message.get_Info().tag);
	assert(rc == true);
	assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);

        printf("Communication complete. Closing connection.\n");
        rc = cif_conn[1]->close();
	assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
	assert(rc == true);
	delete cif_conn[1];  
    }

    printf("Communication complete. Closing connection.\n");
    rc = cif_conn[0]->close();
    assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
    assert(rc == true);
    delete cif_conn[0];

    printf("Closing listener.\n");
    rc = cif_listener[0]->close();
    assert(CIF_Comm::status == CIF_Comm_Status::SUCCESS);
    assert(rc == true);
    delete cif_listener[0];
    CIF_Comm::deactivate();
    exit(0);
}
